@@ -24,7 +24,6 @@ from functools import wraps
 import importlib
 import re
 import base64
-import io
 from hashlib import sha256
 import zlib
 try:
@@ -33,7 +32,7 @@ except ImportError:
     import urllib3
 import os
 import sys
-from copy import deepcopy
+from copy import copy, deepcopy
 import inspect
 from datetime import datetime, timedelta
 from dateutil import parser, tz
@@ -41,25 +40,21 @@ from threading import Lock
 from urllib.parse import urlparse, urljoin, urlencode, parse_qsl
 from uuid import uuid4
 
-try:
-    import requests
-    if int(requests.__version__.split('.')[0]) < 2:
-        raise ImportError()
-except ImportError:
-    raise ImportError('Please install python3-requests >= 2.0')
+import requests
 
 from woob.exceptions import (
     BrowserHTTPSDowngrade, BrowserRedirect, BrowserIncorrectPassword,
+    BrowserUnavailable,
 )
 
 from woob.tools.date import now_as_utc
 from woob.tools.log import getLogger
 from woob.tools.json import json
-from woob import __version__
 
 from .adapters import HTTPAdapter
 from .cookies import WoobCookieJar
 from .exceptions import HTTPNotFound, ClientError, ServerError
+from .har import HARManager
 from .sessions import FuturesSession
 from .profiles import Firefox
 from .pages import NextPage
@@ -153,7 +148,10 @@ class Browser(object):
         self._setup_session(self.PROFILE)
         self.url = None
         self.response = None
-        self.har_bundle = None
+        self.har_manager = None
+
+        if self.responses_dirname is not None:
+            self.har_manager = HARManager(self.responses_dirname, self.logger)
 
     def deinit(self):
         self.session.close()
@@ -165,6 +163,7 @@ class Browser(object):
         if self.responses_dirname is None:
             import tempfile
             self.responses_dirname = tempfile.mkdtemp(prefix='woob_session_')
+            self.har_manager = HARManager(self.responses_dirname, self.logger)
             print('Debug data will be saved in this directory: %s' % self.responses_dirname, file=sys.stderr)
         elif not os.path.isdir(self.responses_dirname):
             os.makedirs(self.responses_dirname)
@@ -217,154 +216,7 @@ class Browser(object):
                 f.write('# %d %s %s\n' % (response.status_code, response.reason, response.headers.get('Content-Type', '')))
                 f.write('%s\t%s\n' % (response.url, filename))
 
-        request = response.request
-
-        if not self.har_bundle:
-            self.har_bundle = {
-                'log': {
-                    'version': '1.2',
-                    'creator': {
-                        'name': 'woob',
-                        'version': __version__,
-                    },
-                    'browser': {
-                        'name': 'woob',
-                        'version': __version__,
-                    },
-                    # there are no pages, but we need that to please firefox
-                    'pages': [{
-                        'id': 'fake_page',
-                        'pageTimings': {},
-                        # and chromium wants some of it too
-                        'startedDateTime': (datetime.now() - response.elapsed).isoformat(),
-                    }],
-                    # don't put additional data after this list, to have a fixed-size suffix after it
-                    # so we can add more entries without rewriting the whole file.
-                    'entries': [],
-                },
-            }
-
-        har_entry = {
-            '$anchor': slug,
-            'startedDateTime': (datetime.now() - response.elapsed).isoformat(),
-            'pageref': 'fake_page',
-            'time': int(response.elapsed.total_seconds() * 1000),
-            'request': {
-                'method': request.method,
-                'url': request.url,
-                'httpVersion': 'HTTP/%.1f' % (response.raw.version / 10.),
-                'headers': [
-                    {
-                        'name': k,
-                        'value': v,
-                    }
-                    for k, v in request.headers.items()
-                ],
-                'queryString': [
-                    {
-                        'name': key,
-                        'value': value,
-                    }
-                    for key, value in parse_qsl(
-                        urlparse(request.url).query,
-                        keep_blank_values=True,
-                    )
-                ],
-                'cookies': [
-                    {
-                        'name': k,
-                        'value': v,
-                    }
-                    for k, v in request._cookies.items()
-                ],
-                # for chromium
-                'bodySize': -1,
-                'headersSize': -1,
-            },
-            'response': {
-                'status': response.status_code,
-                'statusText': response.reason,
-                'httpVersion': 'HTTP/%.1f' % (response.raw.version / 10.),
-                'headers': [
-                    {
-                        'name': k,
-                        'value': v,
-                    }
-                    for k, v in response.headers.items()
-                ],
-                'content': {
-                    'mimeType': response.headers.get('Content-Type', ''),
-                    'size': len(response.content),
-                    # systematically use base64 to avoid more content alteration
-                    # than there already is...
-                    'encoding': "base64",
-                    'text': base64.b64encode(response.content).decode('ascii'),
-                },
-                'cookies': [
-                    {
-                        'name': k,
-                        'value': v,
-                    }
-                    for k, v in response.cookies.items()
-                ],
-                'redirectURL': response.headers.get('location', ''),
-                # for chromium
-                'bodySize': -1,
-                'headersSize': -1,
-            },
-            'timings': {  # please chromium
-                'send': -1,
-                'wait': -1,
-                'receive': -1,
-            },
-            'cache': {},
-        }
-        if request.body is not None:
-            har_entry['request']['postData'] = {
-                'mimeType': request.headers.get('Content-Type', ''),
-                'params': [],
-            }
-            if isinstance(request.body, str):
-                har_entry['request']['postData']['text'] = request.body
-            else:
-                # HAR format has no proper way to encode posted binary data!
-                har_entry['request']['postData']['text'] = request.body.decode('latin-1')
-                # add a non-standard key to indicate how should "text" be decoded.
-                har_entry['request']['postData']['x-binary'] = True
-
-            if request.headers.get('Content-Type') == 'application/x-www-form-urlencoded':
-                har_entry['request']['postData']['params'] = [
-                    {
-                        "name": key,
-                        "value": value,
-                    } for key, value in parse_qsl(request.body)
-                ]
-
-        with self.responses_lock:
-            self.har_bundle['log']['entries'].append(har_entry)
-
-            har_path = os.path.join(self.responses_dirname, 'bundle.har')
-            if not os.path.isfile(har_path):
-                with open(har_path, 'w') as fd:
-                    json.dump(self.har_bundle, fd, separators=(',', ':'))
-            else:
-                # hack to avoid rewriting the whole file: entries are last in the JSON file
-                # we need to seek at the right place and write the new entry.
-                # this will unfortunately overwrite closings.
-                suffix = "]}}"
-                with open(har_path, 'r+') as fd:
-                    # can't seek with a negative value...
-                    fd.seek(0, io.SEEK_END)
-                    after_entry_pos = fd.tell() - len(suffix)
-                    fd.seek(after_entry_pos)
-
-                    if fd.read(len(suffix)) != suffix:
-                        self.logger.warning('HAR file does not end with the expected pattern')
-                    else:
-                        fd.seek(after_entry_pos)
-                        fd.write(',')  # there should have been at least one entry
-                        json.dump(har_entry, fd, separators=(',', ':'))
-                        fd.write(suffix)
+        self.har_manager.save_response(slug, response)
 
         msg = u'Response saved to %s' % response_filepath
         if warning:
@@ -372,6 +224,16 @@ class Browser(object):
         else:
             self.logger.info(msg)
 
+    def _save_request_to_har(self, request):
+        # called when we don't have any response object
+        if not os.path.isdir(self.responses_dirname):
+            os.makedirs(self.responses_dirname)
+
+        slug = uuid4().hex
+        time = self.TIMEOUT * 1000  # because TIMEOUT is in seconds, and we want milliseconds
+        self.har_manager.save_request_only(slug, request, time)
+        self.logger.warning(u'Request saved to %s' % slug)
+
     def _create_session(self):
         return FuturesSession(
             max_workers=self.MAX_WORKERS, max_retries=self.MAX_RETRIES,
@@ -530,15 +392,42 @@ class Browser(object):
             return callback(response)
 
         # call python3-requests
-        response = self.session.send(preq,
-                                     allow_redirects=allow_redirects,
-                                     stream=stream,
-                                     timeout=timeout,
-                                     verify=verify,
-                                     cert=cert,
-                                     proxies=proxies,
-                                     callback=inner_callback,
-                                     is_async=is_async)
+        try:
+            response = self.session.send(preq,
+                                         allow_redirects=allow_redirects,
+                                         stream=stream,
+                                         timeout=timeout,
+                                         verify=verify,
+                                         cert=cert,
+                                         proxies=proxies,
+                                         callback=inner_callback,
+                                         is_async=is_async)
+        except Exception as error:
+            # response in these kind of exception are already stored in HAR
+            # skip them to not store response twice
+            already_handled_exception = any(
+                isinstance(error, exc) for exc in (HTTPNotFound, ClientError, ServerError)
+            )
+            if not already_handled_exception and self.responses_dirname is not None:
+                # when timeout or any kind of exception occur
+                # we don't have any response given by python-requests
+                # but we still need to store request to HAR
+                if isinstance(error, requests.exceptions.RequestException):
+                    if error.response is not None:
+                        slug = uuid4().hex
+                        self.har_manager.save_response(slug, error.response)
+                    elif error.request:
+                        # some exceptions raised by python-requests don't even provide request
+                        # we prefer use error.request because python3-requests may modify request
+                        # by calling add_headers() to it, but it won't happens if
+                        # requests.adapters.HTTPAdapter.add_headers() is not overwritten
+                        self._save_request_to_har(error.request)
+                    else:
+                        # use preq request if the exception doesn't provide the request
+                        self._save_request_to_har(preq)
+                # don't store request for other exceptions, it can be anything
+                # but in that case request has already been stored in HAR file
+            raise
         return response
 
     def async_open(self, url, **kwargs):
@@ -846,6 +735,7 @@ class PagesBrowser(DomainBrowser):
     _urls = None
 
     def __init__(self, *args, **kwargs):
+        self._urls = OrderedDict()
         self.highlight_el = kwargs.pop('highlight_el', False)
         super(PagesBrowser, self).__init__(*args, **kwargs)
 
@@ -859,12 +749,39 @@ class PagesBrowser(DomainBrowser):
         attrs = [(attr, getattr(self, attr)) for attr in dir(self) if not is_property(attr)]
         attrs = [v for v in attrs if isinstance(v[1], URL)]
         attrs.sort(key=lambda v: v[1]._creation_counter)
-        self._urls = OrderedDict(deepcopy(attrs))
-        for k, v in self._urls.items():
+        for k, v in deepcopy(attrs):
+            self._urls[k] = v
             setattr(self, k, v)
         for url in self._urls.values():
             url.browser = self
 
+    def __setattr__(self, key, value):
+        if isinstance(self._urls, OrderedDict):
+            # _urls is instanciated, we can now feed it accordingly.
+            if isinstance(value, URL):
+                # We want to either replace in-place, or add to the URLs.
+                if key in self._urls:
+                    # We want to actually make the old URL unusable.
+                    self._urls[key].browser = None
+
+                value = copy(value)
+                value.browser = self
+                self._urls[key] = value
+            elif key in self._urls:
+                # We want to remove the URL from our mapping only.
+                url = self._urls.pop(key)
+                url.browser = None
+
+        super().__setattr__(key, value)
+
+    def __delattr__(self, key):
+        if isinstance(self._urls, OrderedDict):
+            if key in self._urls:
+                # We want to remove the URL from our mapping.
+                del self._urls[key]
+
+        super().__delattr__(key)
+
     def open(self, *args, **kwargs):
         """
         Same method than
@@ -1243,6 +1160,7 @@ class OAuth2Mixin(StatesMixin):
 
     @property
     def logged(self):
+        # 'access_token_expire' is already set as UTC in OAuth2Mixin.update_token.
         return self.access_token is not None and (not self.access_token_expire or self.access_token_expire > now_as_utc())
 
     def do_login(self):
@@ -1275,17 +1193,35 @@ class OAuth2Mixin(StatesMixin):
         raise BrowserRedirect(self.build_authorization_uri())
 
     def handle_callback_error(self, values):
-        # Here we try to catch callback errors occurring during enrollment
-        # Ideally overload this method in each module to catch specific error
-        assert values.get('code'), "No 'code' was found into the callback url, please raise the right error: %s" % values
+        wrongpass_on_webauth_errors = re.compile('|'.join((
+            'operation canceled by the client',
+            'login cancelled',
+            'consent denied',
+            'psu cancelled the transaction',
+        )))
+
+        error = values.get('error')
+        error_message = values.get('error_description')
+
+        if error == 'access_denied':
+            if error_message:
+                if wrongpass_on_webauth_errors.search(error_message.lower()):
+                    raise BrowserIncorrectPassword(error_message)
+                raise AssertionError(f'Unhandled callback error_message: {error_message}')
+            # access_denied with no error message
+            raise BrowserIncorrectPassword()
+        if error == 'server_error':
+            raise BrowserUnavailable()
+        raise AssertionError(f'Unhandled callback error: {error}, error_message: {error_message}')
 
     def build_access_token_parameters(self, values):
-        return {'code':             values['code'],
-                'grant_type':       'authorization_code',
-                'redirect_uri':     self.redirect_uri,
-                'client_id':        self.client_id,
-                'client_secret':    self.client_secret,
-                }
+        return {
+            'code': values['code'],
+            'grant_type': 'authorization_code',
+            'redirect_uri': self.redirect_uri,
+            'client_id': self.client_id,
+            'client_secret': self.client_secret,
+        }
 
     def do_token_request(self, data):
         return self.open(self.ACCESS_TOKEN_URI, data=data)
@@ -1297,7 +1233,8 @@ class OAuth2Mixin(StatesMixin):
             values = auth_uri
         else:
             values = dict(parse_qsl(urlparse(auth_uri).query))
-        self.handle_callback_error(values)
+        if not values.get('code'):
+            self.handle_callback_error(values)
         self.authorized_date = now_as_utc()
         data = self.build_access_token_parameters(values)
         try:
