@@ -23,16 +23,13 @@ import lxml.html
 
 from woob.exceptions import ParseError
 from woob.tools.log import getLogger, DEBUG_FILTERS
+from woob.tools.misc import NO_DEFAULT as _NO_DEFAULT, NoDefaultType
 
 
 __all__ = ['FilterError', 'ItemNotFound', 'Filter',]
 
-
-class NoDefault(object):
-    def __repr__(self):
-        return 'NO_DEFAULT'
-
-_NO_DEFAULT = NoDefault()
+# Defined for compatibility.
+NoDefault = NoDefaultType
 
 
 class FilterError(ParseError):
@@ -63,15 +60,18 @@ class _Filter(object):
         o.selector = self
         return o
 
+    def __str__(self):
+        return self.__class__.__name__
+
+    def __call__(self, item):
+        raise NotImplementedError()
+
     def default_or_raise(self, exception):
         if self.default is not _NO_DEFAULT:
             return self.default
         else:
             raise exception
 
-    def __str__(self):
-        return self.__class__.__name__
-
     def highlight_el(self, el, item=None):
         obj = self._obj or item
         try:
