@@ -18,8 +18,10 @@
 # along with woob. If not, see <http://www.gnu.org/licenses/>.
 
 
+import enum
 import re
 from datetime import date, datetime
+from typing import Iterable
 
 from unidecode import unidecode
 
@@ -359,6 +361,21 @@ class DebtorAccountRequirement(Enum):
     """Debtor account must not be given"""
 
 
+class Platform(str, enum.Enum):
+    """Mobile platform on which the webview can be run.
+
+    For instance, this enumeration can be used to represent systems on which
+    the authorization link can be catched by a native application instead of
+    the browser.
+    """
+
+    ANDROID = 'android'
+    """Android based platforms."""
+
+    IOS = 'ios'
+    """Apple's iOS platform."""
+
+
 class CapTransfer(Capability):
     can_do_transfer_to_untrusted_beneficiary = False
     """
@@ -378,12 +395,11 @@ class CapTransfer(Capability):
 
     can_do_transfer_cancellation = False
 
-    sca_required_for_transfer_cancellation = True
+    sca_required_for_transfer_cancellation = False
     """
-    The default behaviour is to expect an SCA after sending a transfer
-    cancellation request to validate the request and actually execute the
-    cancellation. If such an SCA is not expected with some banks, the module
-    should set this to False.
+    The default behavior is that we don't need to validate a payment cancellation through a SCA.
+    If a SCA is required after sending a transfer cancellation request to validate it, the module
+    should set this to True.
     """
 
     accepted_beneficiary_types = (BeneficiaryType.RECIPIENT, )
@@ -392,6 +408,38 @@ class CapTransfer(Capability):
     maximum_number_of_instructions = 1
     transfer_with_debtor_account = DebtorAccountRequirement.NOT_USED
 
+    # Indicate that we may not know if the payment is done or rejected, the information
+    # is provided by date type because this behaviour is generally dependent on the type
+    # of payment. An empty list means that the transfer will never get the status ACCEPTED_NO_BANK_STATUS
+    partial_transfer_status_tracking = ()
+
+    is_app_to_app_used_for_transfer = {
+        Platform.ANDROID: None,
+        Platform.IOS: None,
+    }  # type: dict[Platform, bool | None]
+    """
+    Is an App2App flow used for the payment if the PSU has the bank's app installed.
+    None means unknown
+    """
+
+    bank_provides_payer_account = None  # type: bool | None
+    """
+    Once the payment is initiated, does the bank return the payer's account identifier?
+    None means unknown
+    """
+
+    bank_provides_payer_label = None  # type: bool | None
+    """
+    Once the payment is initiated, does the bank return the payer's label?
+    None means unknown
+    """
+
+    transfer_date_types_where_trusted_beneficiary_required = set()  # type: Iterable[TransferDateType]
+    """
+    Set of `TransferDateType` where the beneficiary must be trusted or registered on the payer's banking service.
+    If `iter_transfer_recipients` is implemented, such beneficiaries may be found.
+    """
+
     def iter_transfer_recipients(self, account):
         """
         Iter recipients availables for a transfer from a specific account.
@@ -435,6 +483,35 @@ class CapTransfer(Capability):
         """
         return self.get_transfer(transfer.id)
 
+    def confirm_transfer_cancellation(self, transfer, **params):
+        """
+        Confirm transfer cancellation after a redirect flow.
+
+        :param :class:`Transfer`
+        :rtype: :class:`Transfer`
+        :raises: :class:`AssertionError`: If the payment is not actually cancelled after the whole process
+        """
+        transfer = self.optional_confirm_transfer_cancellation(transfer, **params)
+        # Check that the transfer has been successfully cancelled.
+        if transfer.status != TransferStatus.CANCELLED:
+            raise AssertionError('Transfer is not cancelled after cancellation request.')
+        return transfer
+
+    def optional_confirm_transfer_cancellation(self, transfer, **params):
+        """Proceed with the actual cancellation confirmation.
+
+        This method MUST NOT be called by any external caller. Said caller
+        should actually call confirm_transfer_cancellation which may call the
+        current method if it sees fit.
+
+        The default implementation does not run an explicit confirmation step,
+        it only fetches the up-to-date transfer.
+
+        Modules requiring an explicit cancellation confirmation should
+        overwrite this method, returning the up-to-date transfer at the end.
+        """
+        return self.get_transfer(transfer.id)
+
     def transfer(self, transfer, **params):
         """
         Do a transfer from an account to a recipient.
