@@ -19,9 +19,12 @@
 
 from time import sleep
 
-from .base import Capability, BaseObject, StringField, UserError, BytesField, FloatField, BoolField
+from .base import (
+    BaseObject, BoolField, BytesField, Capability, Field, FloatField,
+    StringField, UserError,
+)
 from ..exceptions import (
-    RecaptchaQuestion, RecaptchaV3Question, RecaptchaV2Question, FuncaptchaQuestion,
+    GeetestV4Question, RecaptchaQuestion, RecaptchaV3Question, RecaptchaV2Question, FuncaptchaQuestion,
     ImageCaptchaQuestion, HcaptchaQuestion,
 )
 
@@ -29,7 +32,7 @@ from ..exceptions import (
 __all__ = [
     'CapCaptchaSolver',
     'SolverJob', 'RecaptchaJob', 'RecaptchaV2Job', 'RecaptchaV3Job',
-    'ImageCaptchaJob', 'HcaptchaJob',
+    'ImageCaptchaJob', 'HcaptchaJob', 'GeetestV4Job',
     'CaptchaError', 'UnsolvableCaptcha', 'InvalidCaptcha', 'InsufficientFunds',
     'exception_to_job',
 ]
@@ -63,6 +66,7 @@ class FuncaptchaJob(SolverJob):
     site_url = StringField('Site URL for FunCaptcha service')
     site_key = StringField('Site key for FunCaptcha service')
     sub_domain = StringField('Required for some complex cases, but Funcaptcha integrations run without it')
+    data = Field('Required for some complex cases', dict)
 
 
 class HcaptchaJob(SolverJob):
@@ -70,6 +74,11 @@ class HcaptchaJob(SolverJob):
     site_key = StringField('Site key for HCaptcha service')
 
 
+class GeetestV4Job(SolverJob):
+    site_url = StringField('Site URL for Geetest service')
+    gt = StringField('Site domain public key')
+
+
 class ImageCaptchaJob(SolverJob):
     image = BytesField('data of the image to solve')
 
@@ -111,6 +120,7 @@ def exception_to_job(exc):
         job.site_url = exc.website_url
         job.site_key = exc.website_key
         job.sub_domain = exc.sub_domain
+        job.data = exc.data
     elif isinstance(exc, ImageCaptchaQuestion):
         job = ImageCaptchaJob()
         job.image = exc.image_data
@@ -118,6 +128,10 @@ def exception_to_job(exc):
         job = HcaptchaJob()
         job.site_url = exc.website_url
         job.site_key = exc.website_key
+    elif isinstance(exc, GeetestV4Question):
+        job = GeetestV4Job()
+        job.site_url = exc.website_url
+        job.gt = exc.gt
     else:
         raise NotImplementedError()
 
