@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright(C) 2010-2014 Romain Bignon, Laurent Bachelier
+# Copyright(C) 2010-2021 Romain Bignon
 #
 # This file is part of woob.
 #
@@ -18,8 +18,8 @@
 # along with woob. If not, see <http://www.gnu.org/licenses/>.
 
 
-from __future__ import print_function
-import imp
+from __future__ import print_function, division
+import importlib
 import posixpath
 import shutil
 import re
@@ -34,26 +34,21 @@ from io import BytesIO, StringIO
 from tempfile import NamedTemporaryFile
 
 from woob.exceptions import BrowserHTTPError, BrowserHTTPNotFound, ModuleInstallError
-from .modules import LoadedModule
 from woob.tools.log import getLogger
 from woob.tools.misc import get_backtrace, to_unicode, find_exe
-from woob.tools.compat import basestring, unicode
 try:
     from ConfigParser import RawConfigParser, DEFAULTSECT
 except ImportError:
     from configparser import RawConfigParser, DEFAULTSECT
 
+from .modules import LoadedModule, _add_in_modules_path
+
 
 @contextmanager
 def open_for_config(filename):
-    if sys.version_info.major == 2:
-        f = NamedTemporaryFile(mode='wb',
-                               dir=os.path.dirname(filename),
-                               delete=False)
-    else:
-        f = NamedTemporaryFile(mode='w', encoding='utf-8',
-                               dir=os.path.dirname(filename),
-                               delete=False)
+    f = NamedTemporaryFile(
+        mode='w', encoding='utf-8', dir=os.path.dirname(filename), delete=False
+    )
     with f:
         yield f
     os.replace(f.name, filename)
@@ -74,20 +69,20 @@ class ModuleInfo(object):
 
         self.version = 0
         self.capabilities = ()
+        self.dependencies = ()
         self.description = u''
         self.maintainer = u''
         self.license = u''
         self.icon = u''
-        self.urls = u''
 
     def load(self, items):
         self.version = int(items['version'])
         self.capabilities = items['capabilities'].split()
+        self.dependencies = items.get('dependencies', '').split()
         self.description = to_unicode(items['description'])
         self.maintainer = to_unicode(items['maintainer'])
         self.license = to_unicode(items['license'])
         self.icon = items['icon'].strip() or None
-        self.urls = items['urls']
 
     def has_caps(self, *caps):
         """Return True if module implements at least one of the caps."""
@@ -109,11 +104,11 @@ class ModuleInfo(object):
     def dump(self):
         return (('version', self.version),
                 ('capabilities', ' '.join(self.capabilities)),
+                ('dependencies', ' '.join(self.dependencies)),
                 ('description', self.description),
                 ('maintainer', self.maintainer),
                 ('license', self.license),
                 ('icon', self.icon or ''),
-                ('urls', self.urls),
                )
 
 
@@ -140,7 +135,7 @@ class Repository(object):
         self.signed = False
         self.key_update = 0
         self.obsolete = False
-        self.logger = getLogger('repository')
+        self.logger = getLogger("%s.repository" % __name__)
         self.errors = {}
 
         self.modules = {}
@@ -193,7 +188,7 @@ class Repository(object):
             try:
                 fp = StringIO(browser.open(posixpath.join(self.url, self.INDEX)).text)
             except BrowserHTTPError as e:
-                raise RepositoryUnavailable(unicode(e))
+                raise RepositoryUnavailable(str(e)) from e
 
         self.parse_index(fp)
         fp.close()
@@ -225,7 +220,7 @@ class Repository(object):
                 keyring_data = browser.open(posixpath.join(self.url, self.KEYRING)).content
                 sig_data = browser.open(posixpath.join(self.url, self.KEYRING + '.sig')).content
             except BrowserHTTPError as e:
-                raise RepositoryUnavailable(unicode(e))
+                raise RepositoryUnavailable(str(e)) from e
             if keyring.exists():
                 if not keyring.is_valid(keyring_data, sig_data):
                     raise InvalidSignature('the keyring itself')
@@ -243,7 +238,7 @@ class Repository(object):
         :type fp: buffer
         """
         config = RawConfigParser()
-        config.readfp(fp)
+        config.read_file(fp)
 
         # Read default parameters
         items = dict(config.items(DEFAULTSECT))
@@ -292,6 +287,8 @@ class Repository(object):
         self.modules.clear()
         self.errors.clear()
 
+        _add_in_modules_path(path)
+
         if os.path.isdir(os.path.join(path, self.KEYDIR)):
             self.signed = True
             self.key_update = self.get_tree_mtime(os.path.join(path, self.KEYDIR), True)
@@ -305,12 +302,8 @@ class Repository(object):
                 continue
 
             try:
-                fp, pathname, description = imp.find_module(name, [path])
-                try:
-                    module = LoadedModule(imp.load_module(name, fp, pathname, description))
-                finally:
-                    if fp:
-                        fp.close()
+                pymodule = importlib.import_module('woob_modules.%s' % name)
+                module = LoadedModule(pymodule)
             except Exception as e:
                 self.logger.warning('Unable to build module %s: [%s] %s' % (name, type(e).__name__, e))
                 bt = get_backtrace(e)
@@ -320,6 +313,7 @@ class Repository(object):
                 m = ModuleInfo(module.name)
                 m.version = self.get_tree_mtime(module_path)
                 m.capabilities = list(set([c.__name__ for c in module.iter_caps()]))
+                m.dependencies = module.dependencies
                 m.description = module.description
                 m.maintainer = module.maintainer
                 m.license = module.license
@@ -364,11 +358,7 @@ class Repository(object):
         for module in self.modules.values():
             config.add_section(module.name)
             for key, value in module.dump():
-                if sys.version_info.major == 2:
-                    # python2's configparser enforces bytes coercion with str(value)...
-                    config.set(module.name, key, to_unicode(value).encode('utf-8'))
-                else:
-                    config.set(module.name, key, value)
+                config.set(module.name, key, value)
 
         with open_for_config(filename) as f:
             config.write(f)
@@ -381,10 +371,11 @@ class Versions(object):
         self.path = path
         self.versions = {}
 
+        config_filename = os.path.join(self.path, self.VERSIONS_LIST)
         try:
-            with open(os.path.join(self.path, self.VERSIONS_LIST), 'r') as fp:
+            with open(config_filename, 'r') as fp:
                 config = RawConfigParser()
-                config.readfp(fp)
+                config.read_file(fp, config_filename)
 
                 # Read default parameters
                 for key, value in config.items(DEFAULTSECT):
@@ -434,6 +425,73 @@ class PrintProgress(IProgress):
         return True
 
 
+class SubProgress(IProgress):
+    def __init__(self, target, steps):
+        self.target = target
+        self.steps = steps
+        self.current = 0
+
+    def progress(self, percent, message):
+        return self.target.progress((self.current + percent) / self.steps, message)
+
+    def __getattr__(self, attr):
+        return getattr(self.target, attr)
+
+
+def recursive_deps(direct_deps, key, result=None):
+    """
+    take a dict of direct dependencies and get all dependencies of an element
+
+    >>> recursive_deps({1: {2, 3}, 2: {3}, 3: {4}, 4: set()}, 1)
+    {2, 3, 4}
+    """
+
+    if result is None:
+        result = set()
+
+    for sub in direct_deps[key]:
+        if sub not in result:
+            result.add(sub)
+            recursive_deps(direct_deps, sub, result)
+    return result
+
+
+class DepList(list):
+    def move_value_after(self, val, afters):
+        # push an element after all its dependencies
+
+        for pos in range(len(self) - 1, -1, -1):
+            if self[pos] == val:
+                # already after
+                return
+            elif self[pos] in afters:
+                break
+        else:
+            raise AssertionError()
+
+        current = self.index(val)
+
+        self.insert(pos + 1, val)
+        del self[current]
+
+
+def dependency_sort(deps_rules):
+    """
+    >>> dependency_sort({1: {2}, 2: {4}, 3: set(), 4: {3}})
+    [3, 4, 2, 1]
+    """
+    # naive but maybe good enough?
+
+    result = DepList(deps_rules)
+    deps_rules = dict(deps_rules)
+
+    for key in deps_rules:
+        deps = recursive_deps(deps_rules, key)
+        result.move_value_after(key, deps)
+
+    return result
+
+
 DEFAULT_SOURCES_LIST = \
 """# List of Woob repositories
 #
@@ -445,13 +503,14 @@ https://updates.woob.tech/%(version)s/main/
 # DEVELOPMENT
 # If you want to hack on Woob modules, you may add a
 # reference to sources, for example:
-#file:///home/rom1/src/woob/modules/
+#file:///home/rom1/src/woob/woob_modules/
 """
 
 
 class Repositories(object):
     SOURCES_LIST = 'sources.list'
     MODULES_DIR = 'modules'
+    MODULES_SUBDIR = 'woob_modules'
     REPOS_DIR = 'repositories'
     KEYRINGS_DIR = 'keyrings'
     ICONS_DIR = 'icons'
@@ -459,7 +518,7 @@ class Repositories(object):
     SHARE_DIRS = [MODULES_DIR, REPOS_DIR, KEYRINGS_DIR, ICONS_DIR]
 
     def __init__(self, workdir, datadir, version):
-        self.logger = getLogger('repositories')
+        self.logger = getLogger("%s.repositories" % __name__)
         self.version = version
 
         self.browser = None
@@ -467,13 +526,14 @@ class Repositories(object):
         self.workdir = workdir
         self.datadir = datadir
         self.sources_list = os.path.join(self.workdir, self.SOURCES_LIST)
-        self.modules_dir = os.path.join(self.datadir, self.MODULES_DIR, self.version)
+        self.modules_dir = os.path.join(self.datadir, self.MODULES_DIR, self.version, self.MODULES_SUBDIR)
         self.repos_dir = os.path.join(self.datadir, self.REPOS_DIR)
         self.keyrings_dir = os.path.join(self.datadir, self.KEYRINGS_DIR)
         self.icons_dir = os.path.join(self.datadir, self.ICONS_DIR)
 
         self.create_dir(self.datadir)
         self.create_dir(self.modules_dir)
+        self.create_namespace_package(self.modules_dir)
         self.create_dir(self.repos_dir)
         self.create_dir(self.keyrings_dir)
         self.create_dir(self.icons_dir)
@@ -490,9 +550,9 @@ class Repositories(object):
             self.load()
 
     def load_browser(self):
+        from urllib.request import getproxies
         from woob.browser.browsers import Browser
         from woob.browser.profiles import Woob as WoobProfile
-        from woob.tools.compat import getproxies
 
         class WoobBrowser(Browser):
             PROFILE = WoobProfile(self.version)
@@ -507,6 +567,20 @@ class Repositories(object):
         elif not os.path.isdir(name):
             self.logger.error(u'"%s" is not a directory' % name)
 
+    namespace_package_content = "from pkgutil import extend_path\n__path__ = extend_path(__path__, __name__)\n"
+
+    def create_namespace_package(self, path):
+        pypath = os.path.join(path, '__init__.py')
+        if os.path.exists(pypath):
+            with open(pypath, "r") as fd:
+                create_file = (fd.read() != self.namespace_package_content)
+        else:
+            create_file = True
+
+        if create_file:
+            with open(pypath, 'wt') as fd:
+                fd.write(self.namespace_package_content)
+
     def _extend_module_info(self, repo, info):
         if repo.local:
             info.path = repo.localurl2path()
@@ -650,6 +724,38 @@ class Repositories(object):
             l.append(repository)
         return True
 
+    def _is_module_updatable(self, info):
+        return not info.is_local() and info.is_installed() and self.versions.get(info.name) != info.version
+
+    def _is_module_installable(self, info):
+        return not info.is_local() and not info.is_installed()
+
+    def _get_all_dependencies(self, modules):
+        modules = list(modules)
+        direct_deps = {}
+
+        i = 0
+        while i < len(modules):
+            current = modules[i]
+
+            for dep_name in current.dependencies:
+                if dep_name in direct_deps:
+                    # already handled
+                    continue
+
+                dep = self.get_module_info(dep_name)
+                if not dep:
+                    raise ModuleInstallError('Module "%s" does not exist' % dep_name)
+                modules.append(dep)
+
+            direct_deps[current.name] = set(current.dependencies)
+
+            i += 1
+
+        # install most depended-on first
+        sorted_names = dependency_sort(direct_deps)
+        return [self.get_module_info(name) for name in sorted_names]
+
     def update(self, progress=PrintProgress()):
         """
         Update repositories and install new packages versions.
@@ -661,29 +767,51 @@ class Repositories(object):
 
         to_update = []
         for name, info in self.get_all_modules_info().items():
-            if not info.is_local() and info.is_installed():
-                if self.versions.get(name) != info.version:
-                    to_update.append(info)
+            if self._is_module_updatable(info):
+                to_update.append(info)
+
+        to_update = self._get_all_dependencies(to_update)
+        to_update = [info for info in to_update if self._is_module_updatable(info) or self._is_module_installable(info)]
 
         if len(to_update) == 0:
             progress.progress(1.0, 'All modules are up-to-date.')
             return
 
-        class InstallProgress(PrintProgress):
-            def __init__(self, n):
-                self.n = n
-
-            def progress(self, percent, message):
-                progress.progress(float(self.n)/len(to_update) + 1.0/len(to_update)*percent, message)
+        proxy_progress = SubProgress(progress, len(to_update))
 
         for n, info in enumerate(to_update):
-            inst_progress = InstallProgress(n)
+            proxy_progress.current = n
             try:
-                self.install(info, inst_progress)
+                self._install_one_module(info, proxy_progress)
             except ModuleInstallError as e:
-                inst_progress.progress(1.0, unicode(e))
+                proxy_progress.progress(1.0, str(e))
 
     def install(self, module, progress=PrintProgress()):
+        if isinstance(module, ModuleInfo):
+            info = module
+        elif isinstance(module, str):
+            progress.progress(0.0, 'Looking for module %s' % module)
+            info = self.get_module_info(module)
+            if not info:
+                raise ModuleInstallError('Module "%s" does not exist' % module)
+        else:
+            raise ValueError('"module" parameter might be a ModuleInfo object or a string, not %r' % module)
+
+        to_install = self._get_all_dependencies([info])
+        to_install = [
+            subinfo
+            for subinfo in to_install
+            if self._is_module_updatable(subinfo)
+            or self._is_module_installable(subinfo)
+        ]
+
+        proxy_progress = SubProgress(progress, len(to_install))
+
+        for n, info in enumerate(to_install):
+            proxy_progress.current = n
+            self._install_one_module(info, proxy_progress)
+
+    def _install_one_module(self, module, progress):
         """
         Install a module.
 
@@ -695,18 +823,6 @@ class Repositories(object):
         import tarfile
         self.load_browser()
 
-        if isinstance(module, ModuleInfo):
-            info = module
-        elif isinstance(module, basestring):
-            progress.progress(0.0, 'Looking for module %s' % module)
-            info = self.get_module_info(module)
-            if not info:
-                raise ModuleInstallError('Module "%s" does not exist' % module)
-        else:
-            raise ValueError('"module" parameter might be a ModuleInfo object or a string, not %r' % module)
-
-        module = info
-
         if module.is_local():
             raise ModuleInstallError('%s is available on local.' % module.name)
 
