@@ -59,14 +59,15 @@ class WoobBase(object):
     :param scheduler: what scheduler to use; default is :class:`woob.core.scheduler.Scheduler`
     :type scheduler: :class:`woob.core.scheduler.IScheduler`
     """
-    VERSION = '3.1'
+    VERSION = '3.2'
 
     def __init__(self, modules_path=None, storage=None, scheduler=None):
         self.logger = getLogger('woob')
         self.backend_instances = {}
         self.requests = RequestsManager()
 
-        self.modules_loader = ModulesLoader(modules_path, self.VERSION)
+        self.modules_path = modules_path
+        self.modules_loader = self.build_modules_loader()
 
         if scheduler is None:
             scheduler = Scheduler()
@@ -84,6 +85,17 @@ class WoobBase(object):
         """
         self.unload_backends()
 
+    def build_modules_loader(self):
+        """
+        Build the module loader for the current application.
+
+        This can be overridden by children to avoid overriding an already
+        existing modules loader.
+
+        :rtype: ModulesLoader
+        """
+        return ModulesLoader(self.modules_path, self.VERSION)
+
     def build_backend(self, module_name, params=None, storage=None, name=None, nofail=False, logger=None):
         """
         Create a backend.
@@ -98,9 +110,9 @@ class WoobBase(object):
         :type storage: :class:`woob.tools.storage.IStorage`
         :param name: name of backend
         :type name: :class:`str`
-        :rtype: :class:`woob.tools.backend.Module`
         :param nofail: if true, this call can't fail
         :type nofail: :class:`bool`
+        :rtype: :class:`woob.tools.backend.Module`
         """
         module = self.modules_loader.get_or_load_module(module_name)
 
@@ -119,7 +131,7 @@ class WoobBase(object):
             super(WoobBase.LoadError, self).__init__(str(exception))
             self.backend_name = backend_name
 
-    def load_backend(self, module_name, name, params=None, storage=None):
+    def load_backend(self, module_name, name, params=None, storage=None, nofail=False):
         """
         Load a backend.
 
@@ -131,6 +143,8 @@ class WoobBase(object):
         :type params: :class:`dict`
         :param storage: storage to use
         :type storage: :class:`woob.tools.storage.IStorage`
+        :param nofail: if true, this call can't fail
+        :type nofail: :class:`bool`
         :rtype: :class:`woob.tools.backend.Module`
         """
         if name is None:
@@ -139,7 +153,7 @@ class WoobBase(object):
         if name in self.backend_instances:
             raise self.LoadError(name, 'A loaded backend already named "%s"' % name)
 
-        backend = self.build_backend(module_name, params, storage, name)
+        backend = self.build_backend(module_name, params, storage, name, nofail=nofail)
         self.backend_instances[name] = backend
         return backend
 
@@ -341,8 +355,6 @@ class Woob(WoobBase):
     BACKENDS_FILENAME = 'backends'
 
     def __init__(self, workdir=None, datadir=None, backends_filename=None, scheduler=None, storage=None):
-        super(Woob, self).__init__(modules_path=False, scheduler=scheduler, storage=storage)
-
         # Create WORKDIR
         xdg_config = Path(os.environ.get("XDG_CONFIG_HOME") or Path.home() / ".config")
 
@@ -370,7 +382,6 @@ class Woob(WoobBase):
 
         # Modules management
         self.repositories = Repositories(workdir, datadir, self.VERSION)
-        self.modules_loader = RepositoryModulesLoader(self.repositories)
 
         # Backend instances config
         if not backends_filename:
@@ -383,6 +394,16 @@ class Woob(WoobBase):
             backends_filename = os.path.join(self.workdir, backends_filename)
         self.backends_config = BackendsConfig(backends_filename)
 
+        super(Woob, self).__init__(modules_path=False, scheduler=scheduler, storage=storage)
+
+    def build_modules_loader(self):
+        """
+        Build the module loader for the current application.
+
+        :rtype: ModulesLoader
+        """
+        return RepositoryModulesLoader(self.repositories)
+
     def _get_working_dir(self, priority_dirs, user_dir, user_dir_legacy):
         for dir in priority_dirs:
             if dir:
