@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright(C) 2010-2012  Christophe Benz, Romain Bignon, Laurent Bachelier
+# Copyright(C) 2010-2021  Christophe Benz, Romain Bignon
 #
 # This file is part of woob.
 #
@@ -25,9 +25,8 @@ import os
 import re
 import shlex
 import signal
-import sys
 from cmd import Cmd
-from collections import OrderedDict
+from collections import OrderedDict, deque
 from datetime import datetime
 from optparse import IndentedHelpFormatter, OptionGroup, OptionParser
 
@@ -36,8 +35,6 @@ from woob.capabilities.collection import BaseCollection, CapCollection, Collecti
 from woob.core import CallErrors
 from woob.exceptions import BrowserQuestion, BrowserRedirect, DecoupledValidation
 from woob.tools.application.formatters.iformatter import MandatoryFieldsNotFound
-from woob.tools.compat import basestring, range, unicode
-from woob.tools.misc import to_unicode
 from woob.tools.path import WorkingPath
 
 from .console import BackendNotGiven, ConsoleApplication
@@ -104,12 +101,7 @@ def defaultcount(default_count=10):
     return deco
 
 
-class MyCmd(Cmd, object):
-    # Hack for Python 2, because Cmd doesn't inherit object, so its __init__ was not called when using super only
-    pass
-
-
-class ReplApplication(ConsoleApplication, MyCmd):
+class ReplApplication(ConsoleApplication, Cmd):
     """
     Base application class for Repl applications.
     """
@@ -187,7 +179,7 @@ class ReplApplication(ConsoleApplication, MyCmd):
         #    of the same line instead of new line.
         #self.prompt = self.BOLD + '%s> ' % self.APPNAME + self.NC
         if len(self.working_path.get()):
-            wp_enc = unicode(self.working_path)
+            wp_enc = str(self.working_path)
             self.prompt = '%s:%s> ' % (self.APPNAME, wp_enc)
         else:
             self.prompt = '%s> ' % (self.APPNAME)
@@ -216,7 +208,7 @@ class ReplApplication(ConsoleApplication, MyCmd):
                 if isinstance(obj, BaseObject):
                     id = obj.fullid
         try:
-            return ConsoleApplication.parse_id(self, id, unique_backend)
+            return super().parse_id(id, unique_backend)
         except BackendNotGiven as e:
             backend_name = None
             while not backend_name:
@@ -273,7 +265,7 @@ class ReplApplication(ConsoleApplication, MyCmd):
         # remove backends that do not have the required method
         new_backend_names = []
         for backend in backend_names:
-            if isinstance(backend, (str, unicode)):
+            if isinstance(backend, str):
                 actual_backend = self.woob.get_backend(backend)
             else:
                 actual_backend = backend
@@ -309,12 +301,12 @@ class ReplApplication(ConsoleApplication, MyCmd):
     def unload_backends(self, *args, **kwargs):
         self.objects = []
         self.collections = []
-        return ConsoleApplication.unload_backends(self, *args, **kwargs)
+        return super().unload_backends(*args, **kwargs)
 
     def load_backends(self, *args, **kwargs):
         self.objects = []
         self.collections = []
-        return ConsoleApplication.load_backends(self, *args, **kwargs)
+        return super(ReplApplication, self).load_backends(*args, **kwargs)
 
     def main(self, argv):
         cmd_args = argv[1:]
@@ -364,7 +356,7 @@ class ReplApplication(ConsoleApplication, MyCmd):
         if backends is None:
             kwargs['backends'] = []
             for backend in self.enabled_backends:
-                actual_function = getattr(backend, function, None) if isinstance(function, basestring) else function
+                actual_function = getattr(backend, function, None) if isinstance(function, str) else function
 
                 if callable(actual_function):
                     kwargs['backends'].append(backend)
@@ -395,6 +387,11 @@ class ReplApplication(ConsoleApplication, MyCmd):
 
         return self.woob.do(self._do_complete, self.options.count, fields, function, *args, **kwargs)
 
+    # TODO make _do_and_retry and _do_and_retry_wait public
+    # TODO (bis): make this comportment as the default of 'do', with support of 'wait()' call on the returned object.
+    def _do_and_retry_wait(self, *args, **kwargs):
+        deque(self._do_and_retry(*args, **kwargs), maxlen=0)
+
     def _do_and_retry(self, *args, **kwargs):
         """
         This method is a wrapper around Woob.do(), and handle interactive
@@ -474,10 +471,7 @@ class ReplApplication(ConsoleApplication, MyCmd):
     # -- command tools ------------
     def parse_command_args(self, line, nb, req_n=None):
         try:
-            if sys.version_info.major >= 3:
-                args = shlex.split(line)
-            else:
-                args = [arg.decode('utf-8') for arg in shlex.split(line.encode('utf-8'))]
+            args = shlex.split(line)
         except ValueError as e:
             raise ArgSyntaxError(str(e))
 
@@ -504,7 +498,7 @@ class ReplApplication(ConsoleApplication, MyCmd):
         """
         This REPL method is overridden to search "short" alias of commands
         """
-        cmd, arg, ignored = Cmd.parseline(self, line)
+        cmd, arg, ignored = super().parseline(line)
 
         if cmd is not None:
             names = set(name for name in self.get_names() if name.startswith('do_'))
@@ -521,7 +515,6 @@ class ReplApplication(ConsoleApplication, MyCmd):
         """
         This REPL method is overridden to catch some particular exceptions.
         """
-        line = to_unicode(line)
         cmd, arg, ignored = self.parseline(line)
 
         # Set the right formatter for the command.
@@ -564,7 +557,7 @@ class ReplApplication(ConsoleApplication, MyCmd):
 
     def default(self, line):
         print('Unknown command: "%s"' % line, file=self.stderr)
-        cmd, arg, ignore = Cmd.parseline(self, line)
+        cmd, arg, ignore = super().parseline(line)
         if cmd is not None:
             names = set(name[3:] for name in self.get_names() if name.startswith('do_' + cmd))
             if len(names) > 0:
@@ -572,7 +565,7 @@ class ReplApplication(ConsoleApplication, MyCmd):
         return os.EX_USAGE
 
     def completenames(self, text, *ignored):
-        return [name for name in Cmd.completenames(self, text, *ignored) if name not in self.hidden_commands]
+        return [name for name in super().completenames(text, *ignored) if name not in self.hidden_commands]
 
     def _shell_completion_items(self):
         items = super(ReplApplication, self)._shell_completion_items()
@@ -837,7 +830,7 @@ class ReplApplication(ConsoleApplication, MyCmd):
         elif action == 'register':
             for name in given_backend_names:
                 instname = self.register_backend(name)
-                if isinstance(instname, basestring):
+                if isinstance(instname, str):
                     self.load_backends(names=[instname])
         elif action == 'edit':
             for backend in given_backends:
@@ -1193,7 +1186,7 @@ class ReplApplication(ConsoleApplication, MyCmd):
                 if len(collections) == 1:
                     self.working_path.split_path = collections[0].split_path
             else:
-                print(u"Path: %s not found" % unicode(self.working_path), file=self.stderr)
+                print(u"Path: %s not found" % self.working_path, file=self.stderr)
                 self.working_path.restore()
                 return 1
 
